/* This program converts a collection of X bitmaps into imf format. */

#include <stdio.h>

char tmpbuf[500];

main(argc, argv)
int argc;
char *argv[];
{
    FILE *ifp;
    int refnum;
    int i, rows, cols, row, col, charcount;
    int numimages;
    char *rawname, *imfname, *xbmname;
    char *data, *mask;

    for (i = 1; i < argc; ++i) {
	rawname = argv[i];
	imfname = rawname;
	xbmname = imfname;
	if ((ifp = fopen(xbmname, "r")) != NULL) {
	    /* also recognize/read pixmaps */
	    ReadBitmapFile(ifp, &cols, &rows, &data);
	    /* also base decision on type of bitmap... */
	    if (1 /* unit (or emblem?) */) {
		if (cols == 16 && rows == 16) {
		    write_16x16(stdout, imfname, cols, rows, data);
		} else {
		    printf("(\"%s\")\n", imfname);
		}
	    }
	    fclose(ifp);
	} else {
	    fprintf(stderr, "Warning: couldn't open \"%s\", ignoring.\n", xbmname);
	}
    }
    exit(0);
}

write_16x16(fp, name, cols, rows, data)
FILE *fp;
int cols, rows;
char *name, *data;
{
    int i, j, row = 0, col, value;
	
    fprintf(fp, "(\"%s\"\n", name);
    fprintf(fp, "  (mono (16 16)\n   \"");
    for (i = 0; i < 16; ++i) {
	if (i < ((16 - rows) / 2) || (16 - i) < ((16 - rows) / 2)) {
	    fprintf(fp, "0000");
	} else {
	    /* crude */
	    /* (ordering probably depends on machine endianness) */
	    fprintf(fp, "%02x", (unsigned char) data[(i * rows + 8) / 8]);
	    fprintf(fp, "%02x", (unsigned char) data[(i * rows + 0) / 8]);
	}
    }
    fprintf(fp, "\"))\n");
}

/* The following code was borrowed from PBM... */

/* xbmtopbm.c - read an X bitmap file and produce a portable bitmap
**
** Copyright (C) 1988 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

#define MAX_LINE 500

ReadBitmapFile( stream, widthP, heightP, dataP )
FILE *stream;
int *widthP, *heightP;
char **dataP;
    {
    char line[MAX_LINE], name_and_type[MAX_LINE];
    char *ptr, *t;
    int version10, raster_length, v;
    register int bytes, bytes_per_line, padding;
    register int c1, c2, value1, value2;
    static int hex_table[256];

    *widthP = *heightP = -1;

    for ( ; ; )
	{
	if ( fgets( line, MAX_LINE, stream ) == NULL )
	    printf( "EOF / read error", 0,0,0,0,0 );
	if ( strlen( line ) == MAX_LINE - 1 )
	    printf( "line too long", 0,0,0,0,0 );

	if ( sscanf( line, "#define %s %d", name_and_type, &v ) == 2 )
	    {
	    if ( ( t = (char *) strrchr( name_and_type, '_' ) ) == NULL )
		t = name_and_type;
	    else
		++t;
	    if ( ! strcmp( "width", t ) )
		*widthP = v;
	    else if ( ! strcmp( "height", t ) )
		*heightP = v;
	    continue;
	    }
	
	if ( sscanf( line, "static short %s = {", name_and_type ) == 1 ) /* } */
	    {
	    version10 = 1;
	    break;
	    }
	if ( sscanf( line, "static char %s = {", name_and_type ) == 1 ) /* } */
	    {
	    version10 = 0;
	    break;
	    }
	}
 
    if ( *widthP == -1 )
	printf( "invalid width", 0,0,0,0,0 );
    if ( *heightP == -1 )
	printf( "invalid height", 0,0,0,0,0 );

    padding = 0;
    if ( ((*widthP % 16) >= 1) && ((*widthP % 16) <= 8) && version10 )
	padding = 1;

    bytes_per_line = (*widthP+7)/8 + padding;
    
    raster_length =  bytes_per_line * *heightP;
    *dataP = (char *) malloc( raster_length );
    if ( *dataP == (char *) 0 )
	printf( "out of memory", 0,0,0,0,0 );

    /* Initialize hex_table. */
    for ( c1 = 0; c1 < 256; ++c1 )
	hex_table[c1] = 256;
    hex_table['0'] = 0;
    hex_table['1'] = 1;
    hex_table['2'] = 2;
    hex_table['3'] = 3;
    hex_table['4'] = 4;
    hex_table['5'] = 5;
    hex_table['6'] = 6;
    hex_table['7'] = 7;
    hex_table['8'] = 8;
    hex_table['9'] = 9;
    hex_table['A'] = 10;
    hex_table['B'] = 11;
    hex_table['C'] = 12;
    hex_table['D'] = 13;
    hex_table['E'] = 14;
    hex_table['F'] = 15;
    hex_table['a'] = 10;
    hex_table['b'] = 11;
    hex_table['c'] = 12;
    hex_table['d'] = 13;
    hex_table['e'] = 14;
    hex_table['f'] = 15;

    if ( version10 )
	for ( bytes = 0, ptr = *dataP; bytes < raster_length; bytes += 2 )
	    {
	    while ( ( c1 = getc( stream ) ) != 'x' )
	        if ( c1 == EOF )
		    printf( "EOF / read error", 0,0,0,0,0 );
	    c1 = getc( stream );
	    c2 = getc( stream );
	    if ( c1 == EOF || c2 == EOF )
		printf( "EOF / read error", 0,0,0,0,0 );
	    value1 = ( hex_table[c1] << 4 ) + hex_table[c2];
	    if ( value1 >= 256 )
		printf( "syntax error", 0,0,0,0,0 );
	    c1 = getc( stream );
	    c2 = getc( stream );
	    if ( c1 == EOF || c2 == EOF )
		printf( "EOF / read error", 0,0,0,0,0 );
	    value2 = ( hex_table[c1] << 4 ) + hex_table[c2];
	    if ( value2 >= 256 )
		printf( "syntax error", 0,0,0,0,0 );
	    *ptr++ = value2;
	    if ( ( ! padding ) || ( ( bytes + 2 ) % bytes_per_line ) )
	        *ptr++ = value1;
	    }
    else
	for ( bytes = 0, ptr = *dataP; bytes < raster_length; ++bytes )
	    {
	    /*
	    ** Skip until digit is found.
	    */
	    for ( ; ; )
		{
		c1 = getc( stream );
		if ( c1 == EOF )
		    printf( "EOF / read error", 0,0,0,0,0 );
		value1 = hex_table[c1];
		if ( value1 != 256 )
		    break;
		}
	    /*
	    ** Loop on digits.
	    */
	    for ( ; ; )
		{
		c2 = getc( stream );
		if ( c2 == EOF )
		    printf( "EOF / read error", 0,0,0,0,0 );
		value2 = hex_table[c2];
		if ( value2 != 256 )
		    {
		    value1 = (value1 << 4) | value2;
		    if ( value1 >= 256 )
			printf( "syntax error", 0,0,0,0,0 );
		    }
		else if ( c2 == 'x' || c2 == 'X' )
		    if ( value1 == 0 )
			continue;
		    else printf( "syntax error", 0,0,0,0,0 );
	        else break;
	        }
	    *ptr++ = value1;
	    }
    }
