/* Inspired by ls, based on xdb by David Curry, davy@riacs.edu. */

#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/keysym.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
/*#include <X11/Xaw/List.h> */
#include <X11/Xaw/Command.h>
#include <X11/Shell.h>

/* We just steal the template widget verbatim to draw text and bitmaps in. */

/* $XConsortium: Template.h,v 1.5 90/12/19 18:46:00 converse Exp $ */

/* Copyright	Massachusetts Institute of Technology	1987, 1988
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef _Template_h
#define _Template_h

/****************************************************************
 *
 * Template widget
 *
 ****************************************************************/

/* Resources:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 destroyCallback     Callback		Pointer		NULL
 height		     Height		Dimension	0
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 sensitive	     Sensitive		Boolean		True
 width		     Width		Dimension	0
 x		     Position		Position	0
 y		     Position		Position	0

*/

/* define any special resource names here that are not in <X11/StringDefs.h> */

#define XtNtemplateResource "templateResource"

#define XtCTemplateResource "TemplateResource"

/* declare specific TemplateWidget class and instance datatypes */

typedef struct _TemplateClassRec*	TemplateWidgetClass;
typedef struct _TemplateRec*		TemplateWidget;

/* declare the class constant */

extern WidgetClass templateWidgetClass;

#endif /* _Template_h */

/* $XConsortium: TemplateP.h,v 1.6 91/03/13 20:12:07 rws Exp $ */

/* Copyright	Massachusetts Institute of Technology	1987, 1988
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef _TemplateP_h
#define _TemplateP_h

#include <X11/Xaw/Template.h>
/* include superclass private header file */
#include <X11/CoreP.h>

/* define unique representation types not found in <X11/StringDefs.h> */

#define XtRTemplateResource "TemplateResource"

typedef struct {
    int empty;
} TemplateClassPart;

typedef struct _TemplateClassRec {
    CoreClassPart	core_class;
    TemplateClassPart	template_class;
} TemplateClassRec;

extern TemplateClassRec templateClassRec;

typedef struct {
    /* resources */
    char* resource;
    /* private state */
} TemplatePart;

typedef struct _TemplateRec {
    CorePart		core;
    TemplatePart	template;
} TemplateRec;

#endif /* _TemplateP_h */

/* Our own definitions start here. */

#include <stdio.h>

#define MAXFILES 1000

char *pname = "xlsb";

char *bitmapfiles[MAXFILES];
int nfiles = 0;

int rows, cols;
int colwid = 100;
int dpywid;

int drawbbox = 0;

Arg tmpargs[10];

XtAppContext app;

Widget toplevel;
Widget mainbox;
Widget tmpwidget;
Widget thelist;

void
shut_down(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    exit(0);
}

void
toggle_frame(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    drawbbox = !drawbbox;
    /* force redisplay */
}

main(argc, argv)
int argc;
char **argv;
{
    int i;

    toplevel = XtAppInitialize(&app, "xlsb",
			       (XrmOptionDescList) NULL, 0, &argc, argv,
			       (String *) NULL, (ArgList) NULL, 0);

    while (--argc) {
	++argv;
	if (!strcmp(*argv, "-bb")) {
	    drawbbox = 1;
	    continue;
	}
	if (!strcmp(*argv, "-frame")) {
	    drawbbox = 1;
	    continue;
	}
	if (!strcmp(*argv, "-px")) {
	    /* set bitmap pixel size */
	    ++argv;
	    continue;
	}
	if (!strcmp(*argv, "-mask")) {
/*	    strcpy(maskext, *(++argv));  */
	    continue;
	}
	if (*argv != NULL) {
	    bitmapfiles[nfiles++] = *argv;
	}
    }
    /* Must have at least one file (could dig through current dir?). */
    if (nfiles == 0) {
	fprintf(stderr, "No bitmaps specified!\n");
	exit(1);
    }
    mainbox = XtCreateManagedWidget("box", boxWidgetClass,
				toplevel, (Arg *) NULL, 0);
    i = 0;
    XtSetArg(tmpargs[i], XtNlabel, "Frame");  ++i;
    tmpwidget =
      XtCreateManagedWidget("frame", commandWidgetClass,
			    mainbox, tmpargs, i);
    XtAddCallback(tmpwidget, XtNcallback, toggle_frame, NULL);
    i = 0;
    XtSetArg(tmpargs[i], XtNlabel, "Quit");  ++i;
    tmpwidget =
      XtCreateManagedWidget("quit", commandWidgetClass,
			    mainbox, tmpargs, i);
    XtAddCallback(tmpwidget, XtNcallback, shut_down, NULL);

    dpywid = 600; /* or some such */
    colwid = 200; /* but should calc from images/string lengths */
    cols = dpywid / colwid;
    /* should adjust cols up/down for extreme cases */
    rows = nfiles / cols + 1;
    cols = nfiles / rows + 1;
    i = 0;
    XtSetArg(tmpargs[i], XtNwidth, dpywid);  ++i;
    XtSetArg(tmpargs[i], XtNheight, 500);  ++i;
    thelist =
      XtCreateManagedWidget("listing", templateWidgetClass,
			    mainbox, tmpargs, i);

    XtRealizeWidget(toplevel);

    XtAppMainLoop(app);
}

usage()
{
    fprintf(stderr, "%s <usual X args> ", pname);
    fprintf(stderr, "[-bb] ... bitmapfiles ...\n");
    exit(1);
}

/* The template widget implementation, plus our hacks. */

/* $XConsortium: Template.c,v 1.4 91/02/17 16:18:42 converse Exp $ */

/* Copyright	Massachusetts Institute of Technology	1987, 1988
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

static XtResource resources[] = {
#define offset(field) XtOffsetOf(TemplateRec, template.field)
    /* {name, class, type, size, offset, default_type, default_addr}, */
    { XtNtemplateResource, XtCTemplateResource, XtRTemplateResource,
	  sizeof(char*), offset(resource), XtRString, (XtPointer) "default" },
#undef offset
};

static void TemplateAction(/* Widget, XEvent*, String*, Cardinal* */);

void redisplay_listing();

static XtActionsRec actions[] =
{
  /* {name, procedure}, */
    {"template",	NULL /* TemplateAction */},
};

static char translations[] =
"<Key>:		template()	\n\
";

TemplateClassRec templateClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"Template",
    /* widget_size		*/	sizeof(TemplateRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	NULL,
    /* initialize_hook		*/	NULL,
    /* realize			*/	XtInheritRealize,
    /* actions			*/	actions,
    /* num_actions		*/	XtNumber(actions),
    /* resources		*/	resources,
    /* num_resources		*/	XtNumber(resources),
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	NULL,
    /* resize			*/	NULL,
    /* expose			*/	redisplay_listing,
    /* set_values		*/	NULL,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	translations,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* template fields */
    /* empty			*/	0
  }
};

WidgetClass templateWidgetClass = (WidgetClass)&templateClassRec;

void
redisplay_listing(widget, evt, region)
Widget widget;
XEvent *evt;
Region region;
{
    int n;

    read_and_layout(widget);
    for (n = 0; n < nfiles; ++n) {
    }
}

int maxwid, maxhgt;

int ws[1000], hs[1000];

/* Read all the bitmaps and compute their desired layout. */

read_and_layout(widget)
Widget widget;
{
    unsigned int x, y, w, h, hotx, hoty, width, height;
    int fontwid, fonthgt;
    int n;
    Pixmap rslt;

#if 0
    font = XLoadQueryFont(dpy, "fixed");
#endif
    fontwid = 10 /* font->max_bounds.width */;
    fonthgt = 15;

    /* Get all the bitmaps and compute sizes. */
    for (n = 0; n < nfiles; ++n) {
	int fullwid;

	if (XReadBitmapFile(XtDisplay(widget), XtWindow(widget),
			    bitmapfiles[n],
			    &width, &height, &rslt, &hotx, &hoty)
	    == BitmapSuccess) {
	    ws[n] = width;  hs[n] = height;
	    if (width > maxwid) maxwid = width;
	    if (height > maxhgt) maxhgt = height;
	    fullwid = width + 2 + fontwid * (strlen(bitmapfiles[n]) + 2);
	    if (fullwid > colwid) colwid = fullwid;
	} else {
	    fprintf(stderr, "%s is not a bitmap file\n", bitmapfiles[n]);
	}
    }
#if 0
    /* Decide on a tasteful layout of the bitmaps. */
    dpywid = 1000 /* XDisplayWidth(dpy, screen)*/;
    if (dpywid > 1000) dpywid = 1000;
    cols = dpywid / colwid;
    /* should adjust cols up/down for extreme cases */
    rows = nfiles / cols + 1;
    cols = nfiles / rows + 1;
#endif
#if 0
    /* This doesn't seem to work... */
    n = 0;
    XtSetArg(resizeargs[n], XtNwidth, cols * colwid);
    ++n;
    XtSetArg(resizeargs[n], XtNheight, rows * (maxhgt + 4));
    ++n;
    XtSetValues(widget, resizeargs, n);
    /* Good a time as any to create the one gc we'll need. */
    /* Can't be shared because we'll be messing with it. */
    {
	XGCValues values;

	values.foreground = 0;
/*	values.font = ?;  */
	gc = XCreateGC(dpy, lswin, GCForeground, &values);
    }
#endif
}

/* Blat a single bitmap+mask+bbox_name onto the window. */

draw_one_bitmap(win, bitmap, name, x, y, width, height)
Window win;
Pixmap bitmap;
char *name;
int x, y;
unsigned int width, height;
{
#if 0
    char *nameproper, *rindex();

    if (bitmap == None) return;

    /* Draw the bounding box if it's been asked for. */
    if (drawbbox) {
	XSetClipMask(dpy, gc, None);
	XSetForeground(dpy, gc, 0);
	XFillRectangle(dpy, win, gc,
		   x + (maxwid - width) / 2 , y + (maxhgt - height) / 2,
		   width, height);
	XSetForeground(dpy, gc, 1);
    }	
    /* Draw the bitmap. */
    XSetClipMask(dpy, gc, bitmap);
    XSetClipOrigin(dpy, gc,
		   x + (maxwid - width) / 2 , y + (maxhgt - height) / 2);
    XFillRectangle(dpy, win, gc,
		   x + (maxwid - width) / 2 , y + (maxhgt - height) / 2,
		   width, height);
    /* Extract the minimal name. */
    if ((nameproper = rindex(name, '/')) == NULL)
      nameproper = name;
    else
      nameproper++;
    /* Draw the name of the bitmap. */
    XSetClipMask(dpy, gc, None);
    /* (need to acquire the right font for this gc) */
    XDrawImageString(dpy, win, lsgc, x + maxwid + 1, y + maxhgt / 2,
		     nameproper, strlen(nameproper));
#endif
}

