/* Copyright (c) 1987, 1988, 1991, 1992  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* The main program of the X11 Xlib version of xconq. */

/* Unlike most multi-player network programs, xconq does not use multiple
   processes.  Instead, it relies on the network capability inherent in
   some graphical interfaces (such as X) to open up multiple displays. */

#include "conq.h"
#include "xconq.h"

/* Just use to skip over stuff. */

XrmOptionDescRec optiondesc[] = {
  { "-bg", "*background", XrmoptionSepArg, NULL }
};

int announced = FALSE;

char *announcemsg = NULL;

/* The main program. */

main(argc, argv)
int argc;
char *argv[];
{
    XrmDatabase rdb = NULL;

    XrmInitialize();
    init_library_path();
    print_any_news();
    /* Fiddle with game module structures. */
    clear_game_modules();
    /* Set up empty data structures. */
    init_data_structures();
    /* Merge command options with resources. */
    XrmParseCommand(&rdb,
		    optiondesc, (sizeof optiondesc / sizeof optiondesc[0]),
		    "Xconq", &argc, argv);
    /* (still need to merge some databases derived from display?) */
    /* Do Unix-style command-line parsing on the remains. */
    parse_command_line(argc, argv, 1);
    load_all_modules();
    check_game_validity();
    parse_command_line(argc, argv, 2);
    set_variants_from_options();
    parse_command_line(argc, argv, 3);
    set_players_from_options();
    make_trial_assignments();
    run_synth_methods();
    final_init();
    assign_players_to_sides();
    run_game(0);
    /* Get the displays set up, but don't draw anything yet. */
    init_all_displays();
    /* Now bring up the init data on each display. */
    init_redraws();
/*    init_sighandlers(); */

    ingame = TRUE;

    while (TRUE) {
	run_game(10);
	get_input(1000);
    }
}

/* The default (human) player is the current user on the current display. */

Player *
add_default_player()
{
    Player *player = add_player();
    
    player->name = getenv("USER");
    player->configname = getenv("XCONQ_CONFIG");
    player->displayname = getenv("DISPLAY");
    return player;
}

/* An init error needs to have the command re-run. */

init_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    fprintf(stderr, "Error: ");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
    exit(1);
}

/* A warning just gets displayed, no other action is taken. */

init_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    fprintf(stderr, "Warning: ");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
}

/* An run error is instantly fatal. */

run_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    /* (should maybe try to save state though?) */
    fprintf(stderr, "Fatal error: ");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    exit(1);
}

/* Runtime warnings are for when it's important to notify some player,
   usually a problem with xconq or a game design. */

run_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    fprintf(stderr, "Warning: ");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
}

do_find_unit() {}
do_find_next_unit() {}
do_next_dead() {}
do_ident() {}

/* Handlers for X catastrophes attempt to do a save first. */
/* Could be more friendly and check success of save, but can't do anything */
/* about it anyway... */

static int
handle_x_error (disp, evt)
Display *disp;
XErrorEvent *evt;
{
    static int num_errors = 0;
    char buf[BUFSIZE];

    XGetErrorText(disp, evt->error_code, buf, BUFSIZE);
    printf("\nX error on display %s: %s\n", DisplayString(disp), buf);
    if (++num_errors >= 10) {
        printf("\nX error: trying emergency save!\n");
        write_entire_game_state("ack!.xconq");
	abort();
    }
}

static int
handle_xio_error (disp)
Display *disp;
{
    printf("\nX IO error on display %s: trying emergency save!\n",
	   DisplayString(disp));
    write_entire_game_state("ack!.xconq");
    abort();
}

#if 0
    XSetIOErrorHandler(handle_xio_error);
    XSetErrorHandler(handle_x_error);
#endif


/* Reading is usually pretty fast, so don't do anything here. */

announce_read_progress() {}

/* Announce the start of a time-consuming computation. */

announce_lengthy_process(msg)
char *msg;
{
    if (n_seconds_elapsed(1)) {
	printf("%s; ", msg);
	fflush(stdout);
	announced = TRUE;
    } else {
	announcemsg = msg;  /* had better be a static string... */
    }
}

announce_progress(percentdone)
int percentdone;
{
    if (n_seconds_elapsed(1)) {
	if (announcemsg) {
	    printf("%s; ", announcemsg);
	    announcemsg = NULL;
	}
	printf(" %d%%", percentdone);
	fflush(stdout);
	announced = TRUE;
    }
}

finish_lengthy_process()
{
    if (announced) {
	printf("; done.\n");
	announced = FALSE;
    }
}

/* Napping is like sleeping, but maybe shorter.  Arg is milliseconds. */

nap(t)
long t;
{
    if (t >= 1000) sleep(t/1000);
}

flush_display_buffers(side)
Side *side;
{
    if (active_display(side)) {
	flush_output(side);
    }
}

/* This routine should be called before any sort of normal exit. */

exit_xconq(side)
Side *side;
{
    /* (should this really happen first?) */
    close_displays();
    /* maybe record in scorefile */
    printf("%s\n", exit_commentary(side));
    exit(0);
}

abort_init()
{
    fprintf(stderr, "Fatal errors encountered during game setup, exiting\n");
    exit(1);
}

close_displays()
{
    Side *side;

    for_all_sides(side) if (active_display(side)) close_display(side);
}

update_everything()
{
    Side *side;

    for_all_sides(side) {
	if (active_display(side)) {
	    /* (update all maps?) */
	}
    }
}

update_event_display()
{
}

update_fire_at_display()
{
}

