/* Copyright (c) 1987, 1988, 1991  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* The main program of the X toolkit version of xconq. */

/* Unlike most multi-player network programs, xconq does not use multiple
   processes.  Instead, it relies on the network capability inherent in
   some graphical interfaces (such as X) to open up multiple displays. */

#include "conq.h"
#include "cmdline.h"
#include "xtconq.h"

extern Options options;

/* Definitions of globally global variables. */

char *possiblegames[100];

char *sideplayers[100];

XtResource appresources[] = {
  { "viewed", "Viewed", XtRBoolean, sizeof(Boolean),
      XtOffsetOf(Options, worldseen), XtRImmediate, (XtPointer) FALSE }
};

/* This list must include all the general xconq options so they'll be left
   alone. */

XrmOptionDescRec optiondesc[] = {
  { "-A", "", XrmoptionNoArg, NULL },
  { "-B", "", XrmoptionNoArg, NULL },
  { "-c", "", XrmoptionSkipArg, NULL },
  { "-D", "", XrmoptionNoArg, (XtPointer) "off" },
  { "-e", "", XrmoptionSkipArg, NULL },
  { "-f", "", XrmoptionSkipArg, NULL },
  { "-g", "", XrmoptionSkipArg, NULL },
  { "-h", "", XrmoptionSkipArg, NULL },
  { "-H", "", XrmoptionNoArg, NULL },
  { "-M", "", XrmoptionSkipArg, NULL },
  { "-timeout", "*timeout", XrmoptionSepArg, NULL },
  { "-world", "*world", XrmoptionSepArg, NULL },
  { "-viewed", "*viewed", XrmoptionNoArg, (XtPointer) "on" }
};

Arg tmpargs[100];

XtAppContext thisapp;

Widget thistoplevel;

Widget choiceshell = NULL;

Widget scenarioshell = NULL;

/* This is the callback that gets done when nobody is doing anything. */

void
handle_timeout(data, id)
XtPointer data;
XtIntervalId *id;
{
    run_game(10);

    if (0 /* real time? */) {
	XtAppAddTimeOut(thisapp, 3000, handle_timeout, NULL);
    }
}

/* The main program calls init stuff, decides which popup to display first,
   then disappears into the Xt app loop forever. */

main(argc, argv)
int argc;
char *argv[];
{
    extern int gamechosen;

    init_library_path();
    print_any_news();
    clear_game_modules();
    init_data_structures();
    thistoplevel =
      XtAppInitialize(&thisapp, "XtConq",
		      optiondesc, XtNumber(optiondesc), &argc, argv,
		      (String *) NULL, (ArgList) NULL, 0);
    XtGetApplicationResources(thistoplevel, (XtPointer) &options,
			      appresources, XtNumber(appresources),
			      (Arg *) NULL, 0);
    parse_command_line(argc, argv, 1);
    init_sighandlers();   /* is this really necessary? */
    if (gamechosen) {
	/* We already chose the game from the cmd line, start setting it up. */
	load_all_modules();
	check_game_validity();
	parse_command_line(argc, argv, 2);
	set_variants_from_options();
	parse_command_line(argc, argv, 3);
	set_players_from_options();
	make_trial_assignments();
	if (1 /* any player/side options left? */) {
	    /* Allow various assignments of players and sides. */
	    popup_scenario_setup();
	} else {
	    /* init from cmd line here, or assume never get here? */
	    /* (this code is out-of-date) */
	    run_synth_methods();
	    assign_players_to_sides();
	    final_init();
	    init_x_displays();
	}
    } else {
	popup_game_choice();
    }
    /* Everything is underway now, stay in here forever. */
    XtAppMainLoop(thisapp);
}

Widget gamelist;

/* Start up a selected game. */

void
go_choice(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    XawListReturnStruct *rslt;
    Module *module;

    rslt = XawListShowCurrent(gamelist);
    if (rslt->list_index != XAW_LIST_NONE) {
	module = create_game_module(rslt->string);
	printf("%s\n", rslt->string);
    } else {
	return;
    }
    load_game_module(module);
    check_game_validity();
/*    set_variants_from_options(); */
    set_players_from_options();
    make_trial_assignments();
    if (1 /* options needing to be set or players to assign */) {
	popup_scenario_setup();
    } else {
	XtPopdown(choiceshell);
	run_synth_methods();
	final_init();
	assign_players_to_sides();
	init_x_displays();
	ingame = TRUE;
	run_game(10);
	if (0) {
	    XtAppAddTimeOut(thisapp, 5000, handle_timeout, NULL);
	}
    }
}

/* Run this if the user quits out of the game choice dialog. */

void
cancel_choice(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    XtPopdown(choiceshell);
    /* Nothing else needs to be done? */
    exit(0);
}

/* Bring up a dialog box that chooses one of several types of game. */

popup_game_choice()
{
    int i, j;
    Widget choicebox, tmpwidget;

    i = 0;
    choiceshell =
      XtCreatePopupShell("ChoicePopup", overrideShellWidgetClass,
			 thistoplevel, tmpargs, i);
    i = 0;
    choicebox =
      XtCreateManagedWidget("ChoiceBox", boxWidgetClass,
			    choiceshell, tmpargs, i);
    i = 0;
    XtSetArg(tmpargs[i], XtNborderWidth, 0);  ++i;
    XtCreateManagedWidget("label", labelWidgetClass,
			  choicebox, tmpargs, i);
    /* (should build from the actual list of games) */
    j = 0;
    possiblegames[j++] = "classic";
    possiblegames[j++] = "standard";
    possiblegames[j++] = "duel";
    possiblegames[j++] = "panzer";
    possiblegames[j++] = "1756";
    i = 0;
    XtSetArg(tmpargs[i], XtNlist, possiblegames);  ++i;
    XtSetArg(tmpargs[i], XtNnumberStrings, j);  ++i;
    XtSetArg(tmpargs[i], XtNdefaultColumns, 1); ++i;
    gamelist =
      XtCreateManagedWidget("GameList", listWidgetClass,
			    choicebox, tmpargs, i);
    i = 0;
    tmpwidget =
      XtCreateManagedWidget("OK", commandWidgetClass,
			    choicebox, tmpargs, i);
    XtAddCallback(tmpwidget, XtNcallback, go_choice, NULL);
    i = 0;
    tmpwidget =
      XtCreateManagedWidget("Cancel", commandWidgetClass,
			    choicebox, tmpargs, i);
    XtAddCallback(tmpwidget, XtNcallback, cancel_choice, NULL);
    XtPopup(choiceshell, XtGrabNone);
}

/* Scenario setup is a dialog with assorted buttons and lists. */

Widget scenariosetup;

/* The assignlist needs to be manipulated. */

Widget assignlist;

void
done_scenario_setup(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    run_synth_methods();
    final_init();
    assign_players_to_sides();
    init_x_displays();
    ingame = TRUE;
    run_game(10);
    if (0) {
	XtAppAddTimeOut(thisapp, 5000, handle_timeout, NULL);
    }
    /* We're in, can get rid of the setup windows. */
    XtPopdown(scenarioshell);
    if (choiceshell != NULL) {
	XtPopdown(choiceshell);
    }
}

/* This callback runs if the setup dialog is cancelled. */

void
cancel_scenario_setup(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    /* Clear everything out so we can start over. */
    clear_game_modules();
    init_data_structures();
    /* Bring this down so we can go back to game choice. */
    XtPopdown(scenarioshell);
    /* Choice dialog might not have been created in the first place,
       if the command line options caused it to be bypassed. */
    if (choiceshell == NULL) {
	popup_game_choice();
    }
}

/* This swaps the selected side or player with the following one. */

void
swap_scenario_setup(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
}

void
edit_scenario_setup(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
}

/* Edit the selected player in the player list. */

void
all_seen_setup(w, clientdata, calldata)
Widget w;
XtPointer clientdata, calldata;
{
    int i;

    options.worldseen != options.worldseen;
    i = 0;
    XtSetArg(tmpargs[i], XtNlabel, (options.worldseen ? "All Seen" : "?"));  ++i;
    XtSetValues(w, tmpargs, i);
}

/* These should be set/grayed according to what has already happened. */

popup_scenario_setup()
{
    int i, j, listlen;
    Widget tmpwidget;
    char *side_name();

    scenarioshell =
      XtCreatePopupShell("ScenarioPopup", overrideShellWidgetClass,
			 thistoplevel, NULL, 0);
    scenariosetup =
      XtCreateManagedWidget("ScenarioSetup", boxWidgetClass,
			    scenarioshell, NULL, 0);
#if 0
    /* World generation controls. */
    i = 0;
    sprintf(spbuf, "%dx%d", area.width, area.height);
    XtSetArg(tmpargs[i], XtNlabel, spbuf);  ++i;
    XtSetArg(tmpargs[i], XtNsensitive, (!terrain_defined()));  ++i;
    XtCreateManagedWidget("world", labelWidgetClass,
			  scenariosetup, tmpargs, i);
    /* Random buttons. */
    i = 0;
    XtSetArg(tmpargs[i], XtNlabel, (options.worldseen ? "All Seen" : "?"));  ++i;
    tmpwidget =
      XtCreateManagedWidget("seen", commandWidgetClass,
			    scenariosetup, tmpargs, i);
    XtAddCallback(tmpwidget, XtNcallback, all_seen_setup, NULL);
#endif

    /* make into update_player_list subr */
    /* make into a one-column tabbed list? */
    for (j = 0; j < g_sides_max(); ++j) {
	Side *side = assignments[j].side;
	Player *player = assignments[j].player;

	/* Very important that these be non-NULL strings. */
	sideplayers[2 * j] =
	  (side ? side_name(side) : "(none)");
	sideplayers[2 * j + 1] =
	  (player ? (player->displayname ? player->displayname : "machine") : "(none)");
    }
    listlen = 2 * j;
    i = 0;
    XtSetArg(tmpargs[i], XtNlist, sideplayers);  ++i;
    XtSetArg(tmpargs[i], XtNnumberStrings, listlen);  ++i;
    XtSetArg(tmpargs[i], XtNverticalList, False); ++i;
    XtSetArg(tmpargs[i], XtNdefaultColumns, 2); ++i;
    assignlist =
      XtCreateManagedWidget("SidePlayerList", listWidgetClass,
			    scenariosetup, tmpargs, i);
    tmpwidget =
      XtCreateManagedWidget("Swap", commandWidgetClass,
			    scenariosetup, NULL, 0);
    XtAddCallback(tmpwidget, XtNcallback, swap_scenario_setup, NULL);
    tmpwidget =
      XtCreateManagedWidget("Done", commandWidgetClass,
			    scenariosetup, NULL, 0);
    XtAddCallback(tmpwidget, XtNcallback, done_scenario_setup, NULL);
    tmpwidget =
      XtCreateManagedWidget("Cancel", commandWidgetClass,
			    scenariosetup, NULL, 0);
    XtAddCallback(tmpwidget, XtNcallback, cancel_scenario_setup, NULL);

    XtPopup(scenarioshell, XtGrabNone);
}

/* These should all be alerts or something. */

/* An init error needs to have the command re-run. */

init_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    fprintf(stderr, "\n");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
    exit(1);
}

/* A warning just gets displayed, no other action is taken. */

init_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    fprintf(stderr, "\n");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
}

/* A run error is fatal. */

run_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    /* (should pop up on all displays) */
    fprintf(stderr, "\n");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    exit(1);
}

/* Runtime warnings are for when it's important to bug the players,
   usually a problem with xconq or a module. */

run_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
    /* (should pop up on all displays) */
    fprintf(stderr, "\n");
    fprintf(stderr, str, a1, a2, a3);
    fprintf(stderr, "\n");
    fflush(stderr);
}

/* Reading is usually pretty fast, so don't do anything here. */

announce_read_progress() {}

/* Announce the start of a time-consuming computation. */
/* Should bring up an alert with text and a percentage bar. */

announce_lengthy_process(msg)
char *msg;
{
    printf("%s;", msg);
    fflush(stdout);
}

announce_progress(percentdone)
int percentdone;
{
    printf(" %d%%", percentdone);
    fflush(stdout);
}

finish_lengthy_process()
{
    printf(" done.\n");
}

update_unit_display(side, unit)
Side *side;
Unit *unit;
{
    /* modify map displays and any unit closeups and any lists */
}

update_side_display(side, side2)
Side *side, *side2;
{
    /* patch side list */
}

flush_display_buffers(side)
Side *side;
{
}

interpret_content_spec() {}

abort_init()
{
    fprintf(stderr, "Fatal errors encountered during game setup, exiting\n");
    exit(1);
}


init_displays()
{
    Side *side;

    for_all_sides(side) {
	if (side_has_display(side)) {
	    init_display(side);
	}
    }
}

close_displays()
{
    Side *side;

    for_all_sides(side) if (active_display(side)) close_display(side);
}

update_action_result_display(side, unit, rslt, rightnow)
Side *side;
Unit *unit;
int rslt, rightnow;
{
    extern HevtDefn hevtdefns[];

    if (active_display(side)) {
/*	notify("%s %s!", unit_desig(unit), hevtdefns[rslt].name);  */
    }
}

update_event_display()
{
}

update_fire_at_display()
{
}


update_everything()
{
    Side *side;

    for_all_sides(side) {
	if (active_display(side)) {
	    /* (update all windows?) */
	}
    }
}

