/* Copyright (c) 1987, 1988, 1991  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* This file implements the various textual displays of xconq. */

#include "conq.h"
#include "xtconq.h"

char tmpnbuf[BUFSIZE];          /* tmp buf for notices only. */

/* Send a message to everybody who's got a screen. */

/*VARARGS*/
notify_all(control, a1, a2, a3, a4, a5, a6)
char *control, *a1, *a2, *a3, *a4, *a5, *a6;
{
    Side *side;

    for_all_sides(side) notify(side, control, a1, a2, a3, a4, a5, a6);
}

/* Main message-sending routine - does its own formatting and spits out to */
/* the given side.  A large chunk of memory is allocated for notices.  Scrolling is by copying pointers to strings. */

/*VARARGS*/
notify(side, control, a1, a2, a3, a4, a5, a6, a7, a8)
Side *side;
char *control, *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
{
    int i;
    static char *note_space = NULL;
    static char *next_string = NULL;

    if (NULL == note_space) {
      note_space = (char *) xmalloc(NOTIFY_SPACE * sizeof(char));
      next_string = note_space;
    }
    if (active_display(side)) {
	for (i = MAXNOTES-1; i > 0; --i) {
	  side->ui->noticebuf[i] =  side->ui->noticebuf[i-1];
	}
	sprintf(tmpnbuf, control, a1, a2, a3, a4, a5, a6, a7, a8);
	/* Capitalize first char of notice. */
	if (islower(tmpnbuf[0])) tmpnbuf[0] = toupper(tmpnbuf[0]);

	/* Following code should be interface-specific. */

	/* Once we are out of space, just reuse it.  It is possible */
	/* that we will clobber old messages, but enough should be */
	/* around that that is not a real problem. */
	   
	if (next_string + strlen(tmpnbuf) + 10 > note_space + NOTIFY_SPACE)
	  next_string = note_space;
	side->ui->noticebuf[0] = next_string;
	/* (should do shortest relative date if dates defined) */
	/* remember date of last note, omit if date unchanged */
	sprintf(side->ui->noticebuf[0], "%d: %s", curturn, tmpnbuf);
	show_note(side);
	flush_output(side);
	while (*next_string++ != '\0');
	side->ui->bottom_note = 0;
	Dprintf("%s\n", side->ui->noticebuf[0]);
    }
}

/* Notice area refresher.  All notes except the most recent one are grayed */
/* out (no effect for monochrome). */

show_note(side)
Side *side;
{
    int i, sy = 0;

    if (active_display(side)) {
	for (i = side->ui->nh+side->ui->bottom_note-1; i >= side->ui->bottom_note; --i) {
	    draw_text(side, side->ui->msg,
		      side->ui->margin, sy, side->ui->noticebuf[i],
		      ((i == 0) ? side->ui->fgcolor : side->ui->graycolor));
	    sy += side->ui->fh;
	}
    }
}

show_all_closeups(side)
Side *side;
{
}

/* Show only the closeups mentioning the given unit. */

show_unit_closeups(side, unit)
Side *side;
Unit *unit;
{
}

show_closeup(side, closeup)
Side *side;
Closeup *closeup;
{
}

/* This routine is a "callback" from the kernel that says something about
   the given unit has changed, so update every display of it. */

update_all_unit_displays(unit)
Unit *unit;
{
    Side *side;
    Closeup *closeup;
    List *list;

    for_all_sides(side) {
	if (active_display(side)) {
	    for_all_closeups(side, closeup) {
		if (alive(unit)) {
		    /* bang the closeup's info */
		} else {
		    XtPopdown(closeup->closeupwin);
		}
	    }
	    for_all_lists(side, list) {
		/* maybe change list entry */
	    }
	    /* may need to change map displays too */
	}
    }
}

char *
describe_side_state(side, side2)
Side *side, *side2;
{
    sprintf(spbuf, "%s (%s)", side2->name, side->player->displayname);
    return spbuf;
}

describe_side(side, side2)
Side *side, *side2;
{
    int u;

    sprintf(spbuf, "Side named \"%s\"", side2->name);
    if (side2->lost) {
	sprintf(spbuf+strlen(spbuf), " (lost)");
    }
    /* next field */
    sprintf(spbuf, "Strategy is under %s control.",
	    (0 /* side2->strategy */ ? "machine" : "no"));
    /* next field */
    sprintf(spbuf, "Ask Side %d, Time Out %d Seconds",
	    side2->unitseveraskside, side2->realtimeout);
    /* per-unit stuff */
    for_all_unit_types(u) {
	sprintf(spbuf, "%s: %s ask side, %s bad terrain,",
		u_type_name(u),
		(u_doctrine(side2, u, everaskside) ? "may" : "never"),
		(u_doctrine(side2, u, avoidbadterrain) ? "avoid" : "ignore")
		);
	sprintf(spbuf, "    build %s.",
		(u_doctrine(side2, u, buildalways) ? "always" : "sometimes"));
    }
}

char *
describe_container(side, unit)
Side *side;
Unit *unit;
{
    if (unit->transport != NULL) {
	sprintf(spbuf, "In %s", short_unit_handle(unit->transport));
    } else {
	/* or describe connection terrain */
	sprintf(spbuf, "In %s",
		t_type_name(terrain_at(unit->x, unit->y)));
    }
    return spbuf;
}

/* Very briefly list the numbers and types of the occupants. */

char *
describe_occupants(side, unit)
Side *side;
Unit *unit;
{
    int u, nums[MAXUTYPES];
    Unit *occ;

    sprintf(spbuf, "");
    if (unit->occupant != NULL) {
	strcpy(spbuf, "Occ ");
	for_all_unit_types(u) nums[u] = 0;
	for_all_occupants(unit, occ) ++nums[occ->type];
	for_all_unit_types(u) {
	    if (nums[u] > 0) {
		sprintf(tmpbuf, "%d %1s  ", nums[u], utype_name_n(u, 1));
		strcat(spbuf, tmpbuf);
	    }
	}
    } else {
	sprintf(spbuf, "No occupants");
    }
    return spbuf;
}

char *
describe_points(side, unit)
Side *side;
Unit *unit;
{
    int u = unit->type;

    /* Describe the "important" parameters like hit points and moves. */
    /* (should say something about parts?) */
    sprintf(spbuf, "HP %d/%d", unit->hp, u_hp(u));
    if (unit->cp < u_cp(u) && u_cp(u) > 0) {
	sprintf(tmpbuf, "  %d%% complete", (unit->cp * 100) / u_cp(u));
	strcat(spbuf, tmpbuf);
    } else {
#if 0
	if (u_acp(u) > 0) {
	    sprintf(tmpbuf, "  Ac %d/%d", unit->actionsleft, u_acp(u));
	    strcat(spbuf, tmpbuf);
	}
	if (u_speed(u) > 0) {
	    sprintf(tmpbuf, "  Mv %d/%d, Actual %d",
		    unit->movesleft, u_speed(u), unit->actualmoves);
	    strcat(spbuf, tmpbuf);
	}
#endif
    }
    return spbuf;
}

char *
describe_supply(side, unit)
Side *side;
Unit *unit;
{
    int r, u = unit->type;

    sprintf(spbuf, "");
    for_all_material_types(r) {
	if (um_storage(u, r) > 0) {
	    sprintf(tmpbuf, "%s %d/%d  ",
		    m_type_name(r), unit->supply[r], um_storage(u, r));
	    strcat(spbuf, tmpbuf);
	}
    }
    return spbuf;
}

char *
describe_plan(side, unit)
Side *side;
Unit *unit;
{
    if (unit->plan) {
	char *pd;

	sprintf(spbuf, "");
	pd = copy_string(plan_desig(unit->plan));
	strcat(spbuf, pd);
    } else {
	sprintf(spbuf, "(mindless)");
    }
    return spbuf;
}

describe_construction(side, unit)
Side *side;
Unit *unit;
{
#if 0
    /* Describe the state of construction. */
    if (building(unit)) {
	sprintf(spbuf, "New %s in ? curturns",
		u_type_name(unit->build->type));
	if (unit->product != unit->next_product) {
	    sprintf(tmpbuf, " (Next prod will be %s)",
		    u_type_name(unit->next_product));
	    strcat(spbuf, tmpbuf);
	}
    } else {
	sprintf(spbuf, "");
    }
#endif
}

describe_stacking(side, unit)
Side *side;
Unit *unit;
{
    int i, nums[MAXUTYPES];
    Unit *occ;

#ifdef STACKING
    sprintf(spbuf, "");
    /* make this a "stacked_at" macro? */
    if ((unit_at(unit->x, unit->y))->nexthere != NULL) {
	strcpy(spbuf, "Others here  ");
	for_all_unit_types(i) nums[i] = 0;
	for_all_stack(unit->x, unit->y, occ) ++nums[occ->type];
	for_all_unit_types(i) {
	    if (nums[i] > 0) {
		sprintf(tmpbuf, "%d %1s  ", nums[i], utype_name_n(i, 1));
		strcat(spbuf, tmpbuf);
	    }
	}
    }
    return spbuf;
#endif /* STACKING */
}

#if 0
#ifdef GEOFEATURES
		sprintf(tmpbuf, "%s", feature_name_at(map->curx, map->cury));
#else
		sprintf(tmpbuf, "");
#endif /* GEOFEATURES */
#ifdef SEASONS
		if (temps_defined()) {
		    sprintf(tmpbuf+strlen(tmpbuf), "(T %d)",
			    avgtemp_at(map->curx, map->cury));
		}
#endif /* SEASONS */
#ifdef ELEVATIONS
		if (elevations_defined()) {
		    sprintf(tmpbuf+strlen(tmpbuf), "(El %d)",
			    elev_at(map->curx, map->cury));
		}
#endif /* ELEVATIONS */
#endif

/* Show a list of all sides in action, drawing a line through any that have */
/* lost out.  This has to be called for each side if everybody's list is */
/* to be updated. */

/* Highlight the side whose turn it is, using reverse video for current */
/* player's own display (to wake she/he/it up) and white instead of gray */
/* for everybody else.  Also add a star for the benefit of monochrome. */

show_game_state(side)
Side *side;
{
    int sx = side->ui->margin + 2 * side->ui->fw, sy = 3 * side->ui->fh, i;
    Side *side2;

    if (active_display(side)) {
/*	show_game_time(side);  */
	show_clock(side);
#if 0
	for_all_sides(side2) {
	    /* Need to work on emblem positioning here */
	    draw_side_emblem(side, side->ui->game,
			     2, sy + 4, -1, side2, TRUE, side->ui->bgcolor);
	    sprintf(tmpbuf, "");
	    if (side2->player->displayname != NULL)
	      sprintf(tmpbuf, "(%s)", short_host_name(side2->player->displayname));
	    sprintf(spbuf, "    : The %s %s",
		    plural_form(side2->name), tmpbuf);
	    draw_text(side, side->ui->game, sx, sy, spbuf,
		      (side == side2 ? side->ui->fgcolor
		       : side->ui->graycolor));
	    /* Draw the line of ignominy through sides that have lost. */
	    if (side2->lost) draw_scratchout(side, sy + side->ui->fh / 2);
	    sy += side->ui->fh;
	}
	/* Draw neutrals. */
	draw_side_emblem(side, side->ui->game,
			 2, sy + 4, -1, NULL, TRUE, side->ui->bgcolor);
	draw_text(side, side->ui->game, sx, sy,
		  "    (neutrals)", side->ui->graycolor);
	/* Indicate whether all the neutrals are gone. */
	/* (Is this a good idea for all scenarios?) */
	if (!side_has_units(indepside)) {
	    draw_scratchout(side, sy + side->ui->fh / 2);
	}
#endif
    }
    /* Draw the "rapidly-changing" stuff. */
    update_sides(side);
}

show_sides() {}

/* The side color here reflects ally/neutral/enemy status. */

side_color(side, side2)
Side *side, *side2;
{
    if (side == side2 || allied_side(side, side2)) return side->ui->altcolor;
    if (enemy_side(side, side2)) return side->ui->enemycolor;
    return side->ui->neutcolor;
}

/* Show which human players still have units to move this turn
   by displaying an asterisk in front of the side name in the sides list. */

/* Also show which sides are currently designers (here because others */
/* should know as soon as possible that someone has acquired great powers). */

update_sides(side)
Side *side;
{
    int sx = side->ui->margin + 2 * side->ui->fw, sy = 3 * side->ui->fh;
    Side *side2;

    if (active_display(side)) {
	for_all_sides(side2) {
	    if (!side2->lost) {
		sprintf(spbuf, "(%d/%d/%d%s)%s",
			side->numwaiting, side->numfinished, side->numacting,
			(side->finishedturn ? "/done" : ""),
			(side->designer ? "!!!" : ""));
		draw_text(side, side->ui->game, sx, sy, spbuf,
			  (side == side2 ? side->ui->fgcolor
			   : side->ui->graycolor));
	    }
	    sy += side->ui->fh;
	}
	flush_output(side);
    }
}

/* Update the time and optional season display. */

update_turn_display(side, rightnow)
Side *side;
int rightnow;
{
    int i;
    extern int paused;

    if (active_display(side)) {
	sprintf(spbuf, "%s", absolute_date_string(curturn));
	i = 0;
	XtSetArg(tmpargs[i], XtNlabel, spbuf);  ++i;
	XtSetValues(side->ui->datelabel, tmpargs, i);
#if 0
#ifdef SEASONS
	if (g_year_length() > 1) {
	    sprintf(tmpbuf, " (%s)", season_name(curturn));
	    strcat(spbuf, tmpbuf);
	}
#endif /* SEASONS */
	draw_fg_text(side, side->ui->game, side->ui->margin, 0, spbuf);
	/* Second line of the game state. */
	sprintf(spbuf, "%s%s",
		(paused ? "Paused" : ""),
		(side->ui->teach ? "*"
		 : (side->ui->followaction ? "L" : " ")));
	draw_fg_text(side, side->ui->game,
		     side->ui->margin, side->ui->fh, spbuf);
	flush_output(side);
#endif
    }
}

/* Each list display is sort of like a directory listing. */

show_all_lists(side)
Side *side;
{
    int u;
    List *list;

    if (active_display(side)) {
	for_all_lists(side, list) {
	    show_list(side, list);
	}
    }
}

/* Fix only the list displays relevant to the given unit. */

update_unit_lists(side, unit)
Side *side;
Unit *unit;
{
    List *list;

    if (active_display(side)) {
	for_all_lists(side, list) {
	    if (list->type == 1) {
		update_unit_list(side, list, unit);
	    }
	}
    }
}

/* Fix only the list displays relevant to the given type. */

update_unit_type_lists(side, u)
Side *side;
int u;
{
    List *list;

    if (active_display(side)) {
	for_all_lists(side, list) {
	    if (list->type == 0) {
		update_unit_type_list(side, list, u);
	    }
	}
    }
}

/* General list display routine. */

show_list(side, list)
Side *side;
List *list;
{
#if 0
    clear_window(side, list->listwin);
    switch (list->type) {
      case 0:
	draw_unit_type_list(side, list);
	break;
      case 1:
	draw_unit_list(side, list);
	break;
    }
#endif
}

/* Draw the entire list of unit types. */

/* (Need a way to draw only the ones present.) */

draw_unit_type_list(side, list)
Side *side;
List *list;
{
#if 0
    int u, pos = 2 * side->ui->fh, spacing = max(hhs[4], side->ui->fh);
    int power = 4;

    /* Draw a header on the list display. */
    draw_fg_text(side, list->listwin, 0, 0,
		 "All Unit Types");
    /* should do a dividing line */
    draw_fg_text(side, list->listwin,
		 hws[power] + side->ui->margin, side->ui->fh,
		 " Num(Bld)     Tot   Lost");
    for_all_unit_types(u) {
	draw_hex_icon(side, list->listwin,
		      side->ui->margin, pos, 0, power, side->ui->bgcolor);
	draw_unit_icon(side, list->listwin,
		       side->ui->margin + (hws[power] - uws[power]) / 2,
		       pos + (hhs[power] - uhs[power]) / 2,
		       u, power, side->ui->fgcolor, side->ui->bgcolor);
	update_unit_type_list(side, list, u);
	pos += spacing;
    }
#endif
}

/* Alter the numbers for a single type of unit.  Should be called right
   after any changes.  Formatted to look nice, but kind of messy to set
   up correctly; display should not jump back and forth as the numbers
   change in size. */

update_unit_type_list(side, list, u)
Side *side;
List *list;
int u;
{
#if 0
    int xpos, ypos;

    if (active_display(side)) {
	/* Tack all the bits of info onto the end of the buffer. */
	sprintf(spbuf, "");
	/* Our unit total (right-justified) */
	if (side->units[u] > 0)	{
	    sprintf(tmpbuf, "%4d", side->units[u]);
	} else {
	    sprintf(tmpbuf, "    ");
	}
	strcat(spbuf, tmpbuf);
	/* Our units under construction (left-justified) */
	if (side->unitsbuilding[u] > 0) {
	    sprintf(tmpbuf, "(%d)", side->unitsbuilding[u]);
	} else {
	    sprintf(tmpbuf, "    ");
	}
	strcat(spbuf, tmpbuf);
	/* Our total possessions over the game */
	/* Make sure that subsequent write goes into valid string area. */
	strcat(spbuf,  "            ");
	if (total_gain(side, u) > 0) {
	    sprintf(tmpbuf, "%4d", total_gain(side, u));
	    sprintf(spbuf+10, "%s", tmpbuf);
	}
	/* Our total losses over the game */
	strcat(spbuf,  "            ");
	if (total_loss(side, u) > 0) {
	    sprintf(tmpbuf, "- %d", total_loss(side, u));
	    sprintf(spbuf+15, "%s", tmpbuf);
	}
	xpos = hws[4] + side->ui->margin;
	ypos = 2 * side->ui->fh
	  + (u) * max(hhs[4], side->ui->fh)
	  + (hhs[4] - side->ui->fh) / 2;
	draw_fg_text(side, list->listwin, xpos, ypos, spbuf);
    }
#endif
}

/* Would be faster to stash these, but enough difference to care? */

total_gain(side, u)
Side *side;
int u;
{
    int i, total = 0;

/*    for (i = 0; i < DUMMYREAS; ++i)
      total += side_balance(side, u, i);  */
    return total;
}

total_loss(side, u)
Side *side;
int u;
{
    int i, total = 0;

/*    for (i = DUMMYREAS+1; i < NUMREASONS; ++i)
      total += side_balance(side, u, i);  */
    return total;
}

/* Unit lists can display all sorts of details. */

update_unit_list_content(side, list)
Side *side;
List *list;
{
    int i = 0, line = 0;
    Unit *unit;
    Side *loopside;

    for_all_units(loopside, unit) {
	if (alive(unit)) {
	    sprintf(spbuf, "%-30s   HP %d/%d",
		    unit_handle(side, unit), unit->hp, u_hp(unit->type));
	} else {
	    sprintf(spbuf, "---");
	}
	list->listbuf[i++] = copy_string(spbuf);
    }
    list->listlen = i;
    i = 0;
    XtSetArg(tmpargs[i], XtNlist, list->listbuf);  ++i;
    XtSetArg(tmpargs[i], XtNnumberStrings, list->listlen);  ++i;
    XtSetValues(list->listwininfo, tmpargs, i);
#if 0
    if (0) {
	draw_fg_text(side, list->listwin, 1, (line++) * side->ui->fh,
		     "Units, in Normal Order");
	for_all_side_units(side, unit) {
	    if (alive(unit)) {
		sprintf(spbuf, "%-30s   HP %d/%d",
			unit_handle(side, unit), unit->hp, u_hp(unit->type));
	    } else {
		sprintf(spbuf, "---");
	    }
	    draw_fg_text(side, list->listwin, 1, (line++) * side->ui->fh,
			 spbuf);
	}
    } else if (1) {
	draw_fg_text(side, list->listwin, 1, (line++) * side->ui->fh,
		     "Units, in Acting Order");
	for (unit = side->actorder; unit != NULL; unit = unit->nextactor) {
	    if (alive(unit)) {
		sprintf(spbuf, "%-30s   HP %d/%d",
			unit_handle(side, unit), unit->hp, u_hp(unit->type));
	    } else {
		sprintf(spbuf, "---");
	    }
	    draw_fg_text(side, list->listwin, 1, (line++) * side->ui->fh,
			 spbuf);
	}
    } else {
	/* special-purpose lists? */
    }
#endif
}

Unit *
find_unit_in_list(side, list, pos)
Side *side;
List *list;
int pos;
{
    int i = 0;
    Unit *unit;
    Side *loopside;

    for_all_units(loopside, unit) {
	if (i == pos)
	  return unit;
	++i;
    }
    return NULL;
}

/* Just change the part of the unit list relevant to the given unit. */

update_unit_list(side, list, unit)
Side *side;
List *list;
Unit *unit;
{
#if 0
    clear_window(side, list->listwin);
    /* will be cleverer eventually */
    draw_unit_list(side, list, unit);
    flush_output(side);
#endif
}

/* Updates to clock need to be sure that display changes immediately. */

update_clock_display(side, rightnow)
Side *side;
int rightnow;
{
    show_clock(side);
/*    flush_output(side); */
}

/* To get the traditional digital clock display, we need fixed-format digit
   printing. */

show_clock(side)
Side *side;
{
    int time = 0 /* from side clock */;
    int hour, minute, second;

    if (active_display(side)) {
	hour = time / 3600;  minute = (time / 60) % 60;  second = time % 60;
	sprintf(spbuf, "%s%d:%s%d:%s%d",
		(hour < 10 ? "0" : ""), hour,
		(minute < 10 ? "0" : ""), minute,
		(second < 10 ? "0" : ""), second);
/*	draw_fg_text(side, side->ui->clock, 0, 0, spbuf);  */
    }
}

/* Dump a file into the help window.  This routine is neither sophisticated */
/* nor robust - lines must be short enough and file must be one page. */
/* Returns success or failure of file opening. */

show_file(side, fname)
Side *side;
char *fname;
{
#if 0
    FILE *fp;

    if ((fp = fopen(fname, "r")) != NULL) {
	while (fgets(spbuf, BUFSIZE, fp)) {
	    spbuf[strlen(spbuf)-1] = '\0';  /* cut off newline */
	    wprintf(side, spbuf);
	}
	fclose(fp);
	return TRUE;
    } else {
	return FALSE;
    }
#endif
}

/* Frequently-called routine to draw text in the foreground color. */

draw_fg_text(side, win, x, y, str)
Side *side;
long win;
int x, y;
char *str;
{
    draw_text(side, win, x, y, str, side->ui->fgcolor);
}


/* Beep the beeper! */

beep(side)
Side *side;
{
    XBell(sdd(side), DefaultScreen(sdd(side)));
}


/* Draw a line through some side's title (indicates that it lost).
   Instead of trying to figure out how long to make it, make it super-long
   and rely on clipping. */

draw_scratchout(side, pos)
Side *side;
int pos;
{
#if 0
    XSetForeground(sdd(side), sd(side)->textgc, side->ui->fgcolor);
    XDrawLine(sdd(side), side->ui->game, sd(side)->textgc,
	      0, pos, display_width(side), pos);
#endif
}

/* General text drawer. */

draw_text() {}

update_action_display(side, rightnow)
Side *side;
int rightnow;
{
}
